﻿using System.Collections.Generic;
using Console.Client.PurolatorWebServiceReference;
using Utilities;

namespace Console.Client
{
	public enum SeverityEnum
	{
		/// <summary>
		/// The call was successful without any warnings or error messages.
		/// </summary>
		Success = 0,
		/// <summary>
		/// The call was successful, but with warnings.
		/// A warning does not prevent the order from being saved, but may indicate a problem may
		/// need to be addressed.  For example, an order with an invalid email address can be created,
		/// but the system won't attempt to send any status updates to the invalid email address.
		/// </summary>
		Warning = 1,
		/// <summary>
		/// The call failed.
		/// An error condition indicates that the Request was invalid in some way and must be
		/// fully corrected before it can be added to OLS.
		/// </summary>
		Error = 2
	}

	class Program
	{
		const string lineBreak = "---------------------------------------------------------------------";

		// TODO: Add valid credentials and printer name.
		static string zplPrinterName = "My ZPL Printer Name";
		static readonly CredentialInfo Credentials = new CredentialInfo { UserName = "", Password = "", ClientID = "0000000" };

		// Just create two orders and close them out.
		static void Main(string[] args)
		{
			List<string> addedPackageIDs = new List<string>();
			const int numberOfOrdersToCreate = 2;
			for (int count = 0; count < numberOfOrdersToCreate; ++count)
				AddOrder(Credentials, addedPackageIDs);

			CloseoutOrder(Credentials, addedPackageIDs);

			System.Console.WriteLine();
			System.Console.WriteLine("Press a key to terminate.");
			System.Console.ReadKey();
		}

		/// <summary>
		/// Get a randomly generated order, and submit it to OLS using web services.
		/// </summary>
		private static void AddOrder(CredentialInfo credentials, List<string> addedPackageIDs)
		{
			// Connection information is stored in the App.config file.
			// The Sandbox connection should be used for testing instead of Production to avoid billing for test packages.
			ShippingServicesV3SoapClient client = new ShippingServicesV3SoapClient("PurolatorWebService");
			// Create a valid order given correct credentials.
			OrderRequestV3 request = ObjectHelper.GetRandomOrderRequest(credentials);

			System.Console.WriteLine(lineBreak);
			System.Console.WriteLine("Sending AddOrder Request.");
			// Attempt to add the order to OLS.
			// If there are no Errors, the label for each package will be marked as printed (even if the
			// label code isn't actually send to the printer).
			// If you want to retain maximum flexibility after submitting an order, then set
			// OrderRequest.LabelType to "None"
			// Update the App.config file if you get a CommunicationException indicating the message is too large.
			OrderResponseV3 response = client.AddOrder(request);

			// Display any errors and display minimal order information.
			DisplayErrorMessages(response);
			DisplayOrderResponseMessage(response, request);
			if (response.Status == (int)SeverityEnum.Error)
				return;

			// Store the PackageIDs so we can close the related packages.
			// This application makes no attempt to keep track of packages created in prior sessions.
			foreach (RequestPackageV3 package in request.Packages)
				addedPackageIDs.Add(package.PackageID);


			// ZPL encoded labels can be sent as raw bytes directly to a ZPL printer (http://www.zebra.com/us/en.html)
			// If OrderRequest.LabelType is IMAGE, then you can (for example) decode it and send it to the printer
			// using a PrintPreviewDialog, or using PrintDocument.
			// (If needed, use PrinterSettings.InstalledPrinters to get a list of installed printers.)

			// To send each ZPL encoded label to a ZPL printer uncomment this code (and update the printer name).
			//foreach (ResponsePackage package in response.Packages)
			//{
			//    // This method sends raw ZPL data to the printer directly.
			//    RawDataPrinterHelper.SendStringToPrinter(zplPrinterName, package.ZPLCode);
			//}
		}

		/// <summary>
		/// Attempt to closeout all shipments created during this session which contain printed labels,
		/// i.e. all packages this application created that were submitted without errors since this application started.
		/// This demo application doesn't attempt to figure out if there are any other packages should be closed out,
		/// and assumes that all packages it successfully creates have had their labels printed.
		/// </summary>
		private static void CloseoutOrder(CredentialInfo credentials, List<string> addedPackageIDs)
		{
			// Connection information (the URL) is stored in the App.config file.
			ShippingServicesV3SoapClient client = new ShippingServicesV3SoapClient("PurolatorWebService");
			// Send array of all packages from this session which haven't yet been closed out.
			CloseoutRequest request = ObjectHelper.GetCloseoutRequest(credentials, addedPackageIDs.ToArray());

			System.Console.WriteLine(lineBreak);
			System.Console.WriteLine("Sending Closeout Request.");
			CloseoutResponse response = client.Closeout(request);

			// Display any errors and indicate which packages we close out (or tried to close out).
			DisplayErrorMessages(response);
			DisplayCloseoutResponseMessage(response, request);

			// Clear the package list if no errors occurred since we can only close them out once.
			if (response.Status != (int)SeverityEnum.Error)
				addedPackageIDs.Clear();
		}

		// We don't need to initialize the ZPL printer if the OrderRequest.LabelType == ZPLIMAGES since ZPLIMAGES
		// includes the encoded ZPL images in each OrderResponse.package[x].ZPLCode instead of just the ZPL for the label.
		// If we use ZPL instead of ZPLIMAGES then the ZPL images will be stored in the printer's memory;
		// when the printer is turned off the images will probably need to be retrieved again.
		// (Some models store the images in flash memory so this may not be necessary for some Zebra printers.)

		///// <summary>
		///// Store the ZPL encoded images in the ZPL printer.  If this is not done before printing a label, the
		///// Purolator label images will be missing.  It does not need to be sent again as long as the printer is on.
		///// This method should be uncommented and used first if any OrderRequests.LabelType will be set to ZPL.
		///// </summary>
		//private static void InitializeZplPrinter()
		//{
		//    System.Console.WriteLine(lineBreak);
		//    if (string.IsNullOrWhiteSpace(zplPrinterName))
		//    {
		//        System.Console.WriteLine(string.Format("Invalid Printer: {0}", zplPrinterName));
		//        return;
		//    }

		//    // Connection information is stored in the App.config file.
		//    ShippingServicesV3SoapClient client = new ShippingServicesV3SoapClient("PurolatorWebService");

		//    //txtResults.Text += string.Format("{0:T}: Sending GetZPLInitializationImages Request.", DateTime.Now);
		//    System.Console.WriteLine("Sending GetZPLInitializationImages Request.");
		//    // Get ZPL encoded Purolator label images - e.g. Purolator logo, 9AM delivery icon, ...
		//    ZPLInitializationResponse response = client.GetZPLInitializationImages(GetCredentials());

		//    if (response.Status == (int)SeverityEnum.Error)
		//    {
		//        DisplayErrorMessages(response);
		//        return;
		//    }

		//    // Send the images to the ZPL printer.  The ZPL printer will save the images internally for later use,
		//    // but will not print anything, flash any lights, or otherwise indicate that anything has been stored.
		//    System.Console.WriteLine("Sending images to printer.");
		//    RawDataPrinterHelper.SendStringToPrinter(zplPrinterName, response.ZPLCode);
		//}

		#region Helper Methods

		/// <summary>
		/// Display any Errors returned by the web service call.
		/// </summary>
		/// <param name="response">The response from the last web service call.</param>
		public static void DisplayErrorMessages(ResponseBase response)
		{
			if (response.Messages == null)
				return;

			foreach (ResponseMessage error in response.Messages)
			{
				// List the error code for each Error and the Error message text.
				if (error.Severity == (int)SeverityEnum.Error)
					System.Console.WriteLine(string.Format("Error ({0}): {1}", error.Number, error.Message));
			}
		}

		/// <summary>
		/// Display the results of the AddOrder.
		/// </summary>
		/// <param name="response">The response from the AddOrder webservice call.</param>
		/// <param name="request">The original request passed to the AddOrder webservice call.</param>
		public static void DisplayOrderResponseMessage(OrderResponseV3 response, OrderRequestV3 request)
		{
			if (response.Status == (int)SeverityEnum.Error)
				System.Console.Write("Attempted to add order: " + request.OrderNumber);
			else
				System.Console.Write("Added order: " + request.OrderNumber);

			System.Console.Write(" Package IDs: ");
			foreach (RequestPackageV3 package in request.Packages)
			{
				System.Console.Write(package.PackageID);
				System.Console.Write(", ");
			}
			System.Console.WriteLine();
		}

		/// <summary>
		/// Display the results of the Closeout call.
		/// </summary>
		/// <param name="response">The response from the Closeout webservice call.</param>
		/// <param name="request">The original request passed to the Closeout webservice call.</param>
		public static void DisplayCloseoutResponseMessage(CloseoutResponse response, CloseoutRequest request)
		{
			if (response.Status == (int)SeverityEnum.Error)
				System.Console.Write("Attempted to closeout packages: ");
			else
				System.Console.Write("Packages closed out: ");

			foreach (string packageID in request.PackageIDs)
			{
				System.Console.Write(packageID);
				System.Console.Write(", ");
			}
			System.Console.WriteLine();
		}

		#endregion
	}
}
