﻿using System.Collections.Generic;
using System.Text;
using System.Windows;
using Utilities;
using WPF.Client.PurolatorWebServiceReference;

namespace WPF.Client
{
	public enum SeverityEnum
	{
		/// <summary>
		/// The call was successful without any warnings or error messages.
		/// </summary>
		Success = 0,
		/// <summary>
		/// The call was successful, but with warnings.
		/// A warning does not prevent the order from being saved, but may indicate a problem may
		/// need to be addressed.  For example, an order with an invalid email address can be created,
		/// but the system won't attempt to send any status updates to the invalid email address.
		/// </summary>
		Warning = 1,
		/// <summary>
		/// The call failed.
		/// An error condition indicates that the Request was invalid in some way and must be
		/// fully corrected before it can be added to OLS.
		/// </summary>
		Error = 2
	}

	public partial class MainWindow
	{
		readonly List<string> addedPackageIDs = new List<string>();
		private const string lineBreak = "---------------------------------------------------------------------\n";
		private const string printerName = "My ZPL Printer Name";

		#region Initialize

		public MainWindow()
		{
			InitializeComponent();
		}

		#endregion

		#region Events

		/// <summary>
		/// Get a randomly generated order, and submit it to OLS using web services.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void OnAddOrderClick(object sender, RoutedEventArgs e)
		{
			// Connection information is stored in the App.config file.
			// The Sandbox connection should be used for testing instead of Production to avoid billing for test packages.
			ShippingServicesV3SoapClient client = new ShippingServicesV3SoapClient("PurolatorWebService");
			// Create a valid order given correct credentials.
			OrderRequestV3 request = ObjectHelper.GetRandomOrderRequest(GetCredentials());

			txtResults.Text += lineBreak;
			txtResults.Text += "Sending AddOrder Request.\n";
			// Attempt to add the order to OLS.
			// If there are no Errors, the label for each package will be marked as printed (even if the
			// label code isn't actually send to the printer).
			// If you want to retain maximum flexibility after submitting an order, then set
			// OrderRequest.LabelType to "None"
			// Update the App.config file if you get a CommunicationException indicating the message is too large.
			OrderResponseV3 response = client.AddOrder(request);

			// Display any errors and display minimal order information.
			DisplayErrorMessages(response);
			DisplayOrderResponseMessage(response, request);
			if (response.Status == (int)SeverityEnum.Error)
				return;

			// Store the PackageIDs so we can close the related packages.
			// This application makes no attempt to keep track of packages created in prior sessions.
			foreach (RequestPackageV3 package in request.Packages)
				addedPackageIDs.Add(package.PackageID);


			// ZPL encoded labels can be sent as raw bytes directly to a ZPL printer (http://www.zebra.com/us/en.html)
			// If OrderRequest.LabelType is IMAGE, then you can (for example) decode it and send it to the printer
			// using a PrintPreviewDialog, or using PrintDocument.
			// (If needed, use PrinterSettings.InstalledPrinters to get a list of installed printers.)

			//// To send each ZPL encoded label to a ZPL printer uncomment this code (and update the printer name).
			//foreach (ResponsePackage package in response.Packages)
			//{
			//    // This method sends raw ZPL data to the printer directly.
			//    RawDataPrinterHelper.SendStringToPrinter(printerName, package.ZPLCode);
			//}
		}

		/// <summary>
		/// Attempt to closeout all shipments created during this session which contain printed labels,
		/// i.e. all packages this application created that were submitted without errors since this application started.
		/// This demo application doesn't attempt to figure out if there are any other packages should be closed out,
		/// and assumes that all packages it successfully creates have had their labels printed.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void OnCloseoutClick(object sender, RoutedEventArgs e)
		{
			// Connection information (the URL) is stored in the App.config file.
			// The Sandbox connection should be used for testing instead of Production to avoid billing for test packages.
			ShippingServicesV3SoapClient client = new ShippingServicesV3SoapClient("PurolatorWebService");
			// Send array of all packages from this session which haven't yet been closed out.
			CloseoutRequest request = ObjectHelper.GetCloseoutRequest(GetCredentials(), addedPackageIDs.ToArray());

			txtResults.Text += lineBreak;
			txtResults.Text += "Sending Closeout Request.\n";
			CloseoutResponse response = client.Closeout(request);

			// Display any errors and indicate which packages we close out (or tried to close out).
			DisplayErrorMessages(response);
			DisplayCloseoutResponseMessage(response, request);

			// Clear the package list if no errors occurred since we can only close them out once.
			if (response.Status != (int)SeverityEnum.Error)
				addedPackageIDs.Clear();
		}

		// We don't need to initialize the ZPL printer if the OrderRequest.LabelType == ZPLIMAGES since ZPLIMAGES
		// includes the encoded ZPL images in each OrderResponse.package[x].ZPLCode instead of just the ZPL for the label.
		// If we use ZPL instead of ZPLIMAGES then the ZPL images will be stored in the printer's memory;
		// when the printer is turned off the images will probably need to be retrieved again.
		// (Some models store the images in flash memory so this may not be necessary for some Zebra printers.)

		///// <summary>
		///// Store the ZPL encoded images in the ZPL printer.  If this is not done before printing a label, the
		///// Purolator label images will be missing.  It does not need to be sent again as long as the printer is on.
		///// This method should be uncommented and used first if any OrderRequests.LabelType will be set to ZPL.
		///// </summary>
		///// <param name="sender"></param>
		///// <param name="e"></param>
		//private void OnInitializeZplPrinterClick(object sender, RoutedEventArgs e)
		//{
		//    txtResults.Text += lineBreak;
		//    if (string.IsNullOrWhiteSpace(printerName))
		//    {
		//        txtResults.Text += string.Format("Invalid Printer: {0}\n", printerName);
		//        return;
		//    }

		//    // Connection information is stored in the App.config file.
		//    ShippingServicesV3SoapClient client = new ShippingServicesV3SoapClient("PurolatorWebService");

		//    //txtResults.Text += string.Format("{0:T}: Sending GetZPLInitializationImages Request.", DateTime.Now);
		//    txtResults.Text += "Sending GetZPLInitializationImages Request.\n";
		//    // Get ZPL encoded Purolator label images - e.g. Purolator logo, 9AM delivery icon, ...
		//    ZPLInitializationResponse response = client.GetZPLInitializationImages(GetCredentials());

		//    if (response.Status == (int)SeverityEnum.Error)
		//    {
		//        DisplayErrorMessages(response);
		//        return;
		//    }

		//    // Send the images to the ZPL printer.  The ZPL printer will save the images internally for later use,
		//    // but will not print anything, flash any lights, or otherwise indicate that anything has been stored.
		//    txtResults.Text += "Sending images to printer.\n";
		//    RawDataPrinterHelper.SendStringToPrinter(printerName, response.ZPLCode);
		//}

		#endregion

		#region Helper Methods

		/// <summary>
		/// Creates a CredentialInfo from the text boxes on the form.
		/// </summary>
		/// <returns>A populated CredentialInfo object.</returns>
		/// <remarks>
		/// The UserName and Password fields must match an active user listed in http://support.pclusa.com/Sandbox/Admin/UserList.aspx
		/// The ClientID must match an active ShipperAccount.  (Purolator support staff typically assigns one ClientID for each facility you ship from.)
		/// </remarks>
		public CredentialInfo GetCredentials()
		{
			CredentialInfo credentials = new CredentialInfo { UserName = txtUsername.Text, Password = txtPassword.Text, ClientID = txtClientID.Text };
			return credentials;
		}

		/// <summary>
		/// Append any Errors returned by the web service call to the results text box.
		/// </summary>
		/// <param name="response">The response from the last web service call.</param>
		public void DisplayErrorMessages(ResponseBase response)
		{
			if (response.Messages == null)
				return;

			StringBuilder fullResults = new StringBuilder();
			fullResults.Append(txtResults.Text);

			foreach (ResponseMessage error in response.Messages)
			{
				// List the error code for each Error and the Error message text.
				if (error.Severity == (int)SeverityEnum.Error)
					fullResults.AppendLine(string.Format("Error ({0}): {1}", error.Number, error.Message));
			}

			txtResults.Text = fullResults.ToString();
		}

		/// <summary>
		/// Append the results of the AddOrder call to the results text box.
		/// </summary>
		/// <param name="response">The response from the AddOrder webservice call.</param>
		/// <param name="request">The original request passed to the AddOrder webservice call.</param>
		public void DisplayOrderResponseMessage(OrderResponseV3 response, OrderRequestV3 request)
		{
			StringBuilder fullResults = new StringBuilder();
			fullResults.Append(txtResults.Text);

			if (response.Status == (int)SeverityEnum.Error)
				fullResults.Append("Attempted to add order: " + request.OrderNumber);
			else
				fullResults.Append("Added order: " + request.OrderNumber);

			fullResults.Append(" Package IDs: ");
			foreach (RequestPackageV3 package in request.Packages)
			{
				fullResults.Append(package.PackageID);
				fullResults.Append(", ");
			}
			fullResults.AppendLine();

			txtResults.Text = fullResults.ToString();
		}

		/// <summary>
		/// Append the results of the Closeout call to the results text box.
		/// </summary>
		/// <param name="response">The response from the Closeout webservice call.</param>
		/// <param name="request">The original request passed to the Closeout webservice call.</param>
		public void DisplayCloseoutResponseMessage(CloseoutResponse response, CloseoutRequest request)
		{
			StringBuilder fullResults = new StringBuilder();
			fullResults.Append(txtResults.Text);

			if (response.Status == (int)SeverityEnum.Error)
				fullResults.Append("Attempted to closeout packages: ");
			else
				fullResults.Append("Packages closed out: ");

			foreach (string packageID in request.PackageIDs)
			{
				fullResults.Append(packageID);
				fullResults.Append(", ");
			}
			fullResults.AppendLine();

			txtResults.Text = fullResults.ToString();
		}

		#endregion
	}
}
